package at.kugel.plugin.dependency;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.SimpleName;
/**
 * Traverse the AST and get all full qualified types used.
 * @author Kugel, <i>Theossos Comp Group</i>
 */
public final class TypeCollectorVisitor extends ASTVisitor
{

   private final Set<String> usedTypes = new HashSet<String>();

   /**
    * Return the list of all names of types used.
    */
   public String[] getUsedTypes()
   {
      final String[] types = usedTypes.toArray(new String[usedTypes.size()]);
      Arrays.sort(types);
      return types;
   }

   private void addName(String astTypeName)
   {
      for (final String fullQualifiedName : astTypeName.split("[<>,\\s\\[\\]]+"))
      {
         usedTypes.add(fullQualifiedName);
      }
   }

   private void addBinding(ITypeBinding astTypeBinding)
   {
      if (astTypeBinding == null)
      {
         return;
      }
      if (astTypeBinding.isPrimitive())
      {
         return;
      }
      addName(astTypeBinding.getQualifiedName());
   }

   // --- visiting ---

   @Override
   public boolean visit(SimpleName node)
   {
      addBinding(node.resolveTypeBinding());
      return super.visit(node);
   }

}
